@@ -106,11 +106,11 @@ modes = [
     },
     {
         'filter': '--1 waiting',
-        'name': 'Queue',
+        'name': 'Queued',
     },
     {
         'filter': '--1 waiting|active',
-        'name': 'Active+Queue',
+        'name': 'Active+Queued',
     },
     {
         'filter': '--1 paused',
@@ -122,17 +122,17 @@ modes = [
     },
     {
         'filter': '--1 error',
-        'name': 'Error',
+        'name': 'Errored',
     },
 ]
 operations_highlights = [
     {
-        "match": "^pause",
+        "match": "^Pause",
         "field": 0,
         "color": 22,
     },
     {
-        "match": "^unpause",
+        "match": "^Resume",
         "field": 0,
         "color": 8,
     },
@@ -171,4 +171,9 @@ operations_highlights = [
         "field": 0,
         "color": 9,
     },
+    {
+        "match": "^Modify",
+        "field": 0,
+        "color": 10,
+    },
 ]
