@@ -30,11 +30,46 @@ from listpick import *
 from listpick.listpick_app import *
 from listpick.ui.keys import *
 
+
+class Operation:
+    def __init__(
+        self,
+        name: str,
+        function: Callable,
+        # function: Callable[curses.window, list[str], list[str]],
+        function_args:dict = {},
+        meta_args: dict = {},
+        exec_only: bool = False,
+        accepts_gids_list: bool = False,
+        send_request: bool = False,
+        view: bool = False,
+        picker_view: bool = False,
+        reapply_terminal_settings = False,
+    ):
+        self.name = name
+        self.function = function
+        self.function_args = function_args
+        self.meta_args = meta_args
+        self.exec_only = exec_only
+        self.accepts_gids_list = accepts_gids_list
+        self.send_request = send_request
+        self.view = view
+        self.picker_view = picker_view
+        self.reapply_terminal_settings = reapply_terminal_settings
+        """
+        Operation.function(
+            stdscr: curses.window,
+            gids: list[str],
+            fnames: list[str],
+
+        )
+        """
+
 def testConnectionFull(url: str = "http://localhost", port: int = 6800) -> bool:
     """ Tests if we can connect to the url and port. """
     url = f'{url}:{port}/jsonrpc'
     try:
-        with rq.urlopen(url, listMethods()) as c:
+        with rq.urlopen(url, listMethods(), timeout=1) as c:
             response = c.read()
         return True
     except:
@@ -45,7 +80,7 @@ def testAriaConnectionFull(url: str = "http://localhost", port: int = 6800) -> b
     url = f'{url}:{port}/jsonrpc'
     try:
         getVersion()
-        with rq.urlopen(url, getVersion()) as c:
+        with rq.urlopen(url, getVersion(), timeout=1) as c:
             response = c.read()
         return True
     except:
@@ -55,7 +90,7 @@ def te(url: str = "http://localhost", port: int = 6800) -> bool:
     """ Tests the connection to the Aria2 server. """
     url = f'{url}:{port}/jsonrpc'
     try:
-        with rq.urlopen(url, listMethods()) as c:
+        with rq.urlopen(url, listMethods(), timeout=1) as c:
             response = c.read()
         return True
     except:
@@ -92,6 +127,7 @@ def getOptionAndFileInfoBatch(gids: list[str]) -> Tuple[list, list]:
     return options_batch, files_info_batch
 
 def dataToPickerRows(dls, options_batch, files_info_batch, show_pc_bar: bool = True):
+    """ Take list of dl dicts and return list of desired attributes along with a header. """
     items = []
     for i, dl in enumerate(dls):
         try:
@@ -118,12 +154,14 @@ def dataToPickerRows(dls, options_batch, files_info_batch, show_pc_bar: bool = T
             size = 0
             for file in files_info['result']:
                 if 'length' in file:
-                    size += int(file['length'])
+                    if 'selected' in file and file['selected'] == "true":
+                        size += int(file['length'])
             # size = int(dl['files'][0]['length'])
             completed = 0
             for file in files_info['result']:
                 if 'completedLength' in file:
-                    completed += int(file['completedLength'])
+                    if 'selected' in file and file['selected'] == "true":
+                        completed += int(file['completedLength'])
             # completed = int(dl['files'][0]['completedLength'])
             pc_complete = completed/size if size > 0 else 0
             pc_bar = convert_percentage_to_ascii_bar(pc_complete*100)
@@ -203,7 +241,7 @@ def printResults(items: list[list[str]], header: list[str]=[]) -> None:
 
 def restartAria() -> None:
     """Restart aria2 daemon."""
-    for cmd in config["general"]["restartcmds"]:
+    for cmd in config["general"]["restart_commands"]:
         subprocess.Popen(cmd, shell=True, stderr=subprocess.PIPE)
     # cmd = f"systemctl --user restart aria2d.service"
     # subprocess.run(cmd, shell=True, stderr=subprocess.PIPE)
@@ -215,7 +253,7 @@ def editConfig() -> None:
     """ Edit the config file in nvim. """
     config =  get_config()
 
-    file = config["general"]["ariaconfigpath"]
+    file = config["general"]["aria2_config_path"]
     cmd = f"nvim {file}"
     process = subprocess.run(cmd, shell=True, stderr=subprocess.PIPE)
 
@@ -229,17 +267,27 @@ def changeOptionDialog(gid:str) -> str:
         return str(e)
 
     with tempfile.NamedTemporaryFile(mode="w", delete=False) as f:
-        json.dump(current_options, f, indent=4)
+        for key, value in current_options.items():
+            f.write(f"{key}={value}\n")
+
         temp_file = f.name
 
-    cmd = f"nvim -i NONE {temp_file}"
+    cmd = f"nvim  -i NONE -c 'set commentstring=#\ %s' {temp_file}"
     subprocess.run(cmd, shell=True)
 
+    loaded_options = {}
     with open(temp_file, "r") as f:
-        loaded_options = json.load(f)
+        for line in f.readlines():
+            line = line.strip()
+            if line.startswith("#"):
+                continue
+            if "=" in line:
+                ind = line.index("=")
+                key, value = line[:ind], line[ind+1:]
+                loaded_options[key.strip()] = value.strip()
 
     # Get difference between dicts
-    keys_with_diff_values = set(key for key in current_options if current_options[key] != loaded_options.get(key, None))
+    keys_with_diff_values = set(key for key in current_options if key in loaded_options and current_options[key] != loaded_options.get(key, None))
 
     reqs = []
     for key in keys_with_diff_values:
@@ -303,17 +351,27 @@ def changeOptionBatchDialog(gids:list) -> str:
         return str(e)
 
     with tempfile.NamedTemporaryFile(mode="w", delete=False) as f:
-        json.dump(current_options, f, indent=4)
+        for key, value in current_options.items():
+            f.write(f"{key}={value}\n")
+
         temp_file = f.name
 
-    cmd = f"nvim -i NONE {temp_file}"
+    cmd = f"nvim -c 'set commentstring=#\ %s' -i NONE {temp_file}"
     subprocess.run(cmd, shell=True)
 
+    loaded_options = {}
     with open(temp_file, "r") as f:
-        loaded_options = json.load(f)
+        for line in f.readlines():
+            line = line.strip()
+            if line.startswith("#"):
+                continue
+            if "=" in line:
+                ind = line.index("=")
+                key, value = line[:ind], line[ind+1:]
+                loaded_options[key.strip()] = value.strip()
 
     # Get difference between dicts
-    keys_with_diff_values = set(key for key in current_options if current_options[key] != loaded_options.get(key, None))
+    keys_with_diff_values = set(key for key in current_options if key in loaded_options and current_options[key] != loaded_options.get(key, None))
 
     reqs = []
     for gid in gids:
@@ -344,7 +402,7 @@ def changeOptionPicker(stdscr: curses.window, gid:str) -> str:
         selected_column=1,
         editable_columns=[False, True],
         keys_dict=edit_menu_keys,
-        startup_notification="'e' to edit cell. 'q' to exit. 'Return' to submit.",
+        startup_notification="'e' to edit cell. 'E' to edit selected cells in nvim. 'q' to exit. 'Return' to submit changes.",
         reset_colours=False,
     )
     selected_indices, opts, function_data = x.run()
@@ -382,10 +440,10 @@ def changeOptionsBatchPicker(stdscr: curses.window, gids:str) -> str:
             items=flattened_json, 
             header=["Key", "Value"],
             title=f"Change Options for {len(gids)} download(s)",
-            sort_column=1,
+            selected_column=1,
             editable_columns=[False, True],
             keys_dict=edit_menu_keys,
-            startup_notification="Press 'e' to edit cell.",
+            startup_notification="'e' to edit cell. 'E' to edit selected cells in nvim. 'q' to exit. 'Return' to submit changes.",
             reset_colours=False,
     )
     selected_indices, opts, function_data = x.run()
@@ -425,7 +483,7 @@ def addUrisFull(url: str ="http://localhost", port: int =6800, token: str = None
     s +=  '# https://docs.python.org/3/_static/py.svg\n#    out=pythonlogo.svg\n#    dir=/home/user/Downloads/\n#    pause=true\n'
     s += '#    user-agent=Mozilla/5.0 (iPhone; CPU iPhone OS 16_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.5 Mobile/15E148 Safari/604.1\n'
     s += '\n'
-    s += "# The full list of DL options can be seen here:\n"
+    s += "# The full list of DL options can be viewed here:\n"
     s += "# https://aria2.github.io/manual/en/html/aria2c.html#input-file\n\n\n"
 
     ## Create tmpfile
@@ -490,6 +548,7 @@ def input_file_lines_to_dict(lines: list[str]) -> Tuple[list[dict], list[str]]:
             exampleimage.iso
             dir=/home/user/images/
         ```
+        returns [{"uri": "http://example.com/image.iso", "dir": "/home/user/images"}], []
     """
 
     downloads = []
@@ -572,7 +631,9 @@ def addTorrentsFull(url: str ="http://localhost", port: int = 6800, token: str =
 
         try:
             jsonreq = addTorrent(dl["path"])
-            sendReq(jsonreq)
+            resp = sendReq(jsonreq)
+            if "result" in resp:
+                gids.append(resp["response"])
             torrent_count += 1
         except:
             pass
@@ -624,6 +685,105 @@ def addTorrentsFilePickerFull(url: str ="http://localhost", port: int = 6800, to
     return gids, f'{torrent_count}/{len(dls)} torrent file(s) added.'
 
 
+def addDownloadsAndTorrentsFull(url: str ="http://localhost", port: int = 6800, token: str =None) -> Tuple[list[str], str]:
+    """
+    Open a kitty prompt to add torrents to Aria2. The file will accept torrent file paths or magnet links and they should be placed on successive lines.
+
+    Example entry for the prompt:
+        ```
+        /home/user/Downloads/torrents/example.torrent
+        magnet:?xt=urn:btih:...
+        ```
+    """
+
+    s = "# Add http(s) links, magnet links, metalinks, or torrent files (by path).\n"
+    s += "# URL\n"
+    s += "#    indented_option=value\n"
+    s += '\n'
+    # s = "!!\n"
+    # s += "# !! arguments inside !! will be applied to all downloads that follow\n"
+    # s += "# !pause=true,queue=0! add and pause, send all to front of queue\n"
+    # s += "# !!argstrings not yet fully implemented\n"
+    s += '# https://docs.python.org/3/_static/py.png\n'
+    s += '# magnet:?xt=urn:btih:...\n'
+    s += "# /path/to/file.torrent\n"
+    s +=  '# https://docs.python.org/3/_static/py.svg\n#    out=pythonlogo.svg\n#    dir=/home/user/Downloads/\n#    pause=true\n'
+    s += '#    user-agent=Mozilla/5.0 (iPhone; CPU iPhone OS 16_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.5 Mobile/15E148 Safari/604.1\n'
+    s += '\n'
+    s += "# The full list of DL options can be viewed here:\n"
+    s += "# https://aria2.github.io/manual/en/html/aria2c.html#input-file\n\n\n"
+
+    ## Create tmpfile
+    with tempfile.NamedTemporaryFile(delete=False, mode='w') as tmpfile:
+        tmpfile.write(s)
+        tmpfile_path = tmpfile.name
+    cmd = f"nvim -i NONE -c 'norm G' {tmpfile_path}"
+    process = subprocess.run(cmd, shell=True, stderr=subprocess.PIPE)
+
+    with open(tmpfile_path, "r") as f:
+        lines = f.readlines()
+
+    dls_list, argstrs = input_file_lines_to_dict(lines)
+
+    valid_keys = input_file_accepted_options
+    dls = []
+    uris = []
+    gids = []
+
+
+    for dl in dls_list:
+        if "uri" not in dl:
+            continue
+
+
+        dl_type = classify_download_string(dl["uri"])
+        if dl_type in ["HTTP", "FTP", "Magnet", "Metalink"]:
+            download_options_dict = {key: val for key,val in dl.items() if key in valid_keys}
+            if "dir" in download_options_dict:
+                download_options_dict["dir"] = os.path.expandvars(os.path.expanduser(download_options_dict["dir"]))
+            uris.append({"uri": dl["uri"], "options": download_options_dict})
+        else:
+            dls.append({"path": os.path.expanduser(dl["uri"])})
+
+    torrent_count = 0
+    for dl in dls:
+
+        try:
+            jsonreq = addTorrent(dl["path"])
+            resp = sendReq(jsonreq)
+            torrent_count += 1
+            if "result" in resp:
+                gids.append(resp["result"])
+        except:
+            pass
+
+
+
+    for dl in uris:
+        uri = dl["uri"]
+        options = dl["options"]
+
+        return_val, gid = addDownload(uri=uri, download_options_dict=options)
+        if return_val: gids.append(gid)
+
+    if len(uris) and torrent_count:
+        msg = f"{len(uris)} direct download(s) added. {torrent_count} torrent(s) added."
+    elif len(uris):
+        msg = f"{len(uris)} direct download(s) added."
+    elif torrent_count:
+        msg = f"{torrent_count} torrent(s) added."
+    else:
+        msg = ""
+    return gids, msg
+
+def addDownloadsAndTorrentsAndPauseFull(url: str ="http://localhost", port: int =6800, token: str = "") -> Tuple[list[str], str]:
+    """ Launch nvim with a tmpfile and allow the user to provide links and/or paths to torrent files. Add them and pause them."""
+    gids, message = addDownloadsAndTorrentsFull(url=url, port=port,token=token)
+    if gids:
+        reqs = [json.loads(pause(gid)) for gid in gids]
+        batch = sendReq(json.dumps(reqs).encode('utf-8'))
+    return gids, f"{len(gids)} download(s) added and paused."
+
 def getAllInfo(gid: str) -> list[dict]:
     """
     Retrieves all information about an aria2 download.
@@ -652,14 +812,22 @@ def retryDownloadFull(gid: str, url: str ="http://localhost", port: int = 6800,
     status = sendReq(tellStatus(gid))
     options = sendReq(getOption(gid))
 
-    uri = status["result"]["files"][0]["uris"][0]["uri"]
-    dl = {
-        "dir": status["result"]["dir"],
-    }
-    dl["out"] = options["result"]["out"] if "out" in options["result"] else ""
-    return_val, gid = addDownload(uri=uri, download_options_dict=dl)
-    if return_val: return gid
-    else: return ""
+    if "bittorrent" not in status["result"]:
+
+        uri = status["result"]["files"][0]["uris"][0]["uri"]
+        dl = {
+            "dir": status["result"]["dir"],
+        }
+        dl["out"] = options["result"]["out"] if "out" in options["result"] else ""
+        return_val, gid = addDownload(uri=uri, download_options_dict=dl)
+        if return_val: return gid
+        else: return ""
+    else:
+        pass
+
+
+
+    return ""
 
 def retryDownloadAndPauseFull(gid: str, url: str ="http://localhost", port: int = 6800, token: str ="") -> None:
     """ Retries a download by getting the options of the existing download and using it to add a new download and then pauses the download. Does not remove the old download. Returns the gid of the new download or an empty string if there is an error. """
@@ -668,7 +836,7 @@ def retryDownloadAndPauseFull(gid: str, url: str ="http://localhost", port: int
 
 
 
-def getAll(items, header, visible_rows_indices, getting_data):
+def getAll(items, header, visible_rows_indices, getting_data, state):
     """ Retrieves all downloads: active, stopped, and queue. Also returns the header. """
     active, aheader = getActive()
     stopped, sheader = getStopped()
@@ -823,76 +991,101 @@ def openFiles(files: list[str]) -> None:
         subprocess.Popen(f"gio launch /usr/share/applications/{app} {files_str}", shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
 
 
-def applyToDownloads(stdscr: curses.window, gids: list = [], operation_name: str = "", operation_function: Callable = lambda:None, operation_function_args: dict = {}, user_opts: str = "", view: bool =False, fnames:list=[], picker_view: bool = False) -> None:
+def applyToDownloads(
+    stdscr: curses.window,
+    operation: Operation,
+    gids: list = [],
+    operation_name: str = "",
+    operation_function: Callable = lambda:None,
+    operation_function_args: dict = {},
+    user_opts: str = "",
+    view: bool =False,
+    fnames:list=[],
+    picker_view: bool = False
+) -> None:
+    """
+    Applies a given operation to a list of aria2c GIDs.
+
+    Parameters:
+    - stdscr: The standard screen object from curses.
+    - operation: An object containing details about the operation to perform.
+    - gids: A list of download IDs to which the operation will be applied.
+    - operation_name: Optional name for the operation.
+    - operation_function: Optional function to handle the operation logic.
+    - operation_function_args: Optional arguments for the operation function.
+    - user_opts: User-provided options, such as position for change operations.
+    - view: Flag indicating whether to display the results in a view.
+    - fnames: List of filenames corresponding to the gids.
+    - picker_view: Flag indicating whether to use a picker view for displaying results.
+
+    Returns: None
+    """
 
     responses = []
-    if len(gids) ==0 : return None
-    if operation_name in ["Open File(s) (do not group)", "Open File(s)"]:
-        operation_function(gids)
-    elif operation_name in ["Change Options nvim (for all selected)"]:
-        operation_function(gids)
-    elif operation_name in ["Change Options Picker (for each selected)"]:
-        operation_function(stdscr, gids[0])
-    elif operation_name in ["Change Options Picker (for all selected)"]:
-        operation_function(stdscr, gids)
-    elif operation_name == "Transfer Speed Graph *experimental*":
-        fname = fnames[0]
-        if len(fname)>20: fname = fname[:17] + '...'
-
-        operation_function_args["title"] = f"{repr(fname)} Transfer Speeds"
-        operation_function(gid=str(gids[0]), **operation_function_args)
+    if len(gids) == 0 : return None
 
+    result = []
+    if operation.accepts_gids_list:
+        result = operation.function(
+            stdscr=stdscr, 
+            gids=gids,
+            fnames=fnames,
+            operation=operation,
+            function_args=operation.function_args
+        )
+        if operation.send_request:
+            result = sendReq(result)
     else:
-        for gid in gids:
+        for i, gid in enumerate(gids):
             try:
                 jsonreq = {}
-                if operation_name == "Change Position":
+                if operation.name == "Change Position":
                     position = int(user_opts) if user_opts.strip().isdigit() else 0
-                    jsonreq = operation_function(str(gid), position)
-                elif operation_name == "DL Info: Get All Info":
-                    js_rs = getAllInfo(str(gid))
-                    responses.append(js_rs)
-                elif operation_name == "Send to Back of Queue":
-                    jsonreq = operation_function(str(gid), pos=10000)
-                elif operation_name == "Send to Front of Queue":
-                    jsonreq = operation_function(str(gid), pos=0)
-                # elif len(operation_list) > 2:
-                #     operation_kwargs = operation_list[2]
-                #     jsonreq = operation_function(str(gid), **operation_kwargs)
+                    result_part = changePosition(gid, pos=position)
                 else:
-                    jsonreq = operation_function(gid=str(gid), **operation_function_args)
+                    result_part = operation.function(
+                        stdscr=stdscr, 
+                        gid=gid,
+                        fname=fnames[i],
+                        operation=operation,
+                        function_args=operation.function_args
+                    )
 
-                js_rs = sendReq(jsonreq)
-                responses.append(js_rs)
+                if operation.send_request:
+                    result_part = sendReq(result_part)
+                result.append(result_part)
             except:
                 pass
-                # responses.append({})
-        if view:
-            with tempfile.NamedTemporaryFile(delete=False, mode='w') as tmpfile:
-                for i, response in enumerate(responses):
-                    tmpfile.write(f'{"*"*50}\n{str(i)+": "+gids[i]:^50}\n{"*"*50}\n')
-                    tmpfile.write(json.dumps(response, indent=4))
-                tmpfile_path = tmpfile.name
-            # cmd = r"nvim -i NONE -c '/^\s*\"function\"'" + f" {tmpfile_path}"
-            # cmd = r"""nvim -i NONE -c 'setlocal bt=nofile' -c 'silent! %s/^\s*"function"/\0' -c 'norm ggn'""" + f" {tmpfile_path}"
-            cmd = f"nvim {tmpfile_path}"
-            process = subprocess.run(cmd, shell=True, stderr=subprocess.PIPE)
-        elif picker_view:
-            l = []
+
+
+    if operation.picker_view:
+        l = []
+        for i, response in enumerate(result):
+            l += [[gid, "------"]]
+            if "result" in response: response = response["result"]
+            response = process_dl_dict(response)
+            l += [[key, val] for key, val in flatten_data(response).items()]
+        x = Picker(
+            stdscr,
+            items=l,
+            search_query="function",
+            title=operation.name,
+            header=["Key", "Value"],
+            reset_colours=False,
+            cell_cursor=False,
+        )
+        x.run()
+    elif operation.view:
+        with tempfile.NamedTemporaryFile(delete=False, mode='w') as tmpfile:
             for i, response in enumerate(responses):
-                l += [[gid, "------"]]
-                if "result" in response: response = response["result"]
-                response = process_dl_dict(response)
-                l += [[key, val] for key, val in flatten_data(response).items()]
-            x = Picker(
-                stdscr,
-                items=l,
-                search_query="function",
-                title=operation_name,
-                header=["Key", "Value"],
-                reset_colours=False,
-            )
-            x.run()
+                tmpfile.write(f'{"*"*50}\n{str(i)+": "+gids[i]:^50}\n{"*"*50}\n')
+                tmpfile.write(json.dumps(result, indent=4))
+            tmpfile_path = tmpfile.name
+        # cmd = r"nvim -i NONE -c '/^\s*\"function\"'" + f" {tmpfile_path}"
+        # cmd = r"""nvim -i NONE -c 'setlocal bt=nofile' -c 'silent! %s/^\s*"function"/\0' -c 'norm ggn'""" + f" {tmpfile_path}"
+        cmd = f"nvim -c 'set commentstring=#\ %s' {tmpfile_path}"
+        process = subprocess.run(cmd, shell=True, stderr=subprocess.PIPE)
+
 
     stdscr.clear()
 
@@ -905,6 +1098,72 @@ def process_dl_dict(dls):
                 dl[key] = bytes_to_human_readable(dl[key])
     return dls
 
+def download_selected_files(stdscr, gids):
+    """
+    Present the user with files for each given GID and allow them to select which files should be downloaded.
+
+    Args:
+        stdscr (ncurses.window): The main window for ncurses application.
+        gids (list): A list of group IDs for which files are to be selected.
+
+    Returns:
+        None
+    """
+
+    for gid in gids:
+        req = getFiles(gid)
+        files_dict = sendReq(req)["result"]
+        options = sendReq(getOption(gid))
+        dir = options["result"]["dir"]
+
+        # files = [os.path.basename(f["path"]) for f in files_dict]
+        files = [f["path"].replace(dir, "") for f in files_dict]
+        sizes = [bytes_to_human_readable(f["length"]) for f in files_dict]
+        selected_indices = [i for i in range(len(files_dict)) if files_dict[i]['selected'] == 'true']
+        selections = {i: f['selected'] == "true" for i, f in enumerate(files_dict)}
+        items = [[files[i], sizes[i]] for i in range(len(files))]
+        header = ["File", "Size"]
+        
+        # from listpick.ui.keys import picker_keys as pk
+        # from copy import copy
+        # pk = copy(pk)
+        # pk["edit"] = [ord('e')]
+
+        selectionsPicker = Picker(
+            stdscr,
+            items=items,
+            header=header,
+            selections=selections,
+            cell_cursor=False,
+            editable_columns=[True, False],
+            editable_by_default=True,
+            keys_dict=picker_keys,
+            startup_notification="Selected files will be downloaded. Non-selected will be skipped. 'e' to edit filename. 'E' to edit selected cells in nvim. 'q' to exit. 'Return' to submit changes.",
+        )
+        modified_selections, options, function_data = selectionsPicker.run()
+        if selected_indices != modified_selections and function_data["last_key"] != ord("q"):
+            selected = ",".join([str(x+1) for x in modified_selections])
+            try:
+                js_req = changeOption(gid, "select-file", selected)
+                resp = sendReq(js_req)
+            except:
+                pass
+        filename_changes = False
+        for i, row in enumerate(selectionsPicker.items):
+            if files[i] == row[0]: continue
+            # If the values differ then a name has been changed
+
+            filename_changes = True
+            js_req = changeOption(gid, "index-out", f"{i+1}={row[0]}")
+            resp = sendReq(js_req)
+        if filename_changes:
+            js_req = changeOption(gid, "check-integrity", "true")
+            resp = sendReq(js_req)
+
+
+
+
+
 def getGlobalSpeed() -> str:
     resp = sendReq(getGlobalStat())
     up = bytes_to_human_readable(resp['result']['uploadSpeed'])
@@ -915,7 +1174,21 @@ def getGlobalSpeed() -> str:
     return f"{down}/s 󰇚 {up}/s 󰕒 | {numActive}A {numWaiting}W {numStopped}S"
     return f"{down}/s 󰇚  {up}/s 󰕒"
         
-def bytes_to_human_readable(size: float) -> str:
+def bytes_to_human_readable(size: float, sep =" ", round_at=1) -> str:
+    """
+    Convert a number of bytes to a human readable string.
+
+    size (int): the number of bytes
+    sep (str): the string that should separate the size from the units.
+                A single space by default.
+    round_at (int): the unit below which the figure should be rounded
+            round_at=0:  0.0B, 23.1KB, 2.3MB
+            round_at=1:  0B, 23.1KB, 2.3MB
+            round_at=2:  0B, 23KB, 2.3MB
+
+    Examples:
+    1024000 -> 1 MB
+    """
     suffixes = ['B', 'KB', 'MB', 'GB', 'TB']
     if isinstance(size, str):
         size=float(size)
@@ -923,7 +1196,11 @@ def bytes_to_human_readable(size: float) -> str:
     while size >= 1024 and i < len(suffixes)-1:
         size /= 1024.0
         i += 1
-    return f"{size:.1f} {suffixes[i]}"
+    if i < round_at:
+        size_str = f"{int(size)}"
+    else:
+        size_str = f"{size:.1f}"
+    return f"{size_str}{sep}{suffixes[i]}"
 
 def get_config(path="") -> dict:
     """ Get config from file. """
@@ -935,16 +1212,34 @@ def get_config(path="") -> dict:
         if os.path.exists(os.path.expanduser(os.environ["ARIA2TUI_CONFIG_PATH"])):
             CONFIGPATH = os.environ["ARIA2TUI_CONFIG_PATH"]
 
+    ## Ensure that users with old keys in their config are not bothered by key changes
+    new_keys_to_old = {
+        "startup_commands": "startupcmds",
+        "restart_commands": "restartcmds",
+        "aria2_config_path": "ariaconfigpath",
+    }
+    old_keys_to_new = {
+        "startupcmds": "startup_commands",
+        "restartcmds": "restart_commands",
+        "ariaconfigpath": "aria2_config_path"
+    }
+
     if os.path.exists(os.path.expanduser(CONFIGPATH)):
         with open(os.path.expanduser(CONFIGPATH), "r") as f:
-            config = toml.load(f)
+            user_config = toml.load(f)
 
-        if "general" in config:
-            for key in config["general"]:
-                full_config["general"][key] = config["general"][key]
-        if "appearance" in config:
-            for key in config["appearance"]:
-                full_config["appearance"][key] = config["appearance"][key]
+        if "general" in user_config:
+            for user_key in user_config["general"]:
+                full_config_key = user_key
+                if user_key in old_keys_to_new:
+                    full_config_key = old_keys_to_new[user_key]
+                full_config["general"][full_config_key] = user_config["general"][user_key]
+        if "appearance" in user_config:
+            for user_key in user_config["appearance"]:
+                full_config_key = user_key
+                if user_key in old_keys_to_new:
+                    full_config_key = old_keys_to_new[user_key]
+                full_config["appearance"][full_config_key] = user_config["appearance"][user_key]
 
     return full_config
 
@@ -954,9 +1249,9 @@ def get_default_config() -> dict:
             "url": "http://localhost",
             "port": "6800",
             "token": "",
-            "startupcmds": ["aria2c"],
-            "restartcmds": ["pkill aria2c && sleep 1 && aria2c"],
-            "ariaconfigpath": "~/.config/aria2/aria2.conf",
+            "startup_commands": ["aria2c"],
+            "restart_commands": ["pkill aria2c && sleep 1 && aria2c"],
+            "aria2_config_path": "~/.config/aria2/aria2.conf",
             "paginate": False,
             "refresh_timer": 2,
             "global_stats_timer": 1,
@@ -972,6 +1267,32 @@ def get_default_config() -> dict:
     }
     return default_config
 
+def classify_download_string(input_string: str) -> str:
+    magnet_link_pattern = r'^magnet:\?xt=urn:btih'
+    metalink_pattern = r'^metalink:'
+    ftp_pattern = r'^(ftp|ftps|sftp)://'
+    http_pattern = r'^(http|https)://'
+
+    # Check if the input string matches any of the patterns
+    if re.match(magnet_link_pattern, input_string):
+        return "Magnet"
+    elif re.match(metalink_pattern, input_string):
+        return "Metalink"
+    elif re.match(ftp_pattern, input_string):
+        return "FTP"
+    elif re.match(http_pattern, input_string):
+        return "HTTP"
+
+    # Check if the input string is a file path
+    if os.path.exists(os.path.expanduser(os.path.expandvars(input_string))) and os.path.isfile(input_string) and input_string.endswith(".torrent"):
+        return "Torrent File"
+
+    return ""
+
+
+
+
+
 # default = get_default_config()
 config = get_config()
 url = config["general"]["url"]
@@ -1015,5 +1336,8 @@ addTorrents = lambda url=url, port=port, token=token: addTorrentsFull(url=url, p
 addTorrentsFilePicker = lambda url=url, port=port, token=token: addTorrentsFilePickerFull(url=url, port=port, token=token)
 addUris = lambda url=url, port=port, token=token: addUrisFull(url=url, port=port, token=token)
 addUrisAndPause = lambda url=url, port=port, token=token: addUrisAndPauseFull(url=url, port=port, token=token)
+addDownloadsAndTorrents = lambda url=url, port=port, token=token: addDownloadsAndTorrentsFull(url=url, port=port, token=token)
+addDownloadsAndTorrentsAndPause = lambda url=url, port=port, token=token: addDownloadsAndTorrentsAndPauseFull(url=url, port=port, token=token)
 testConnection = lambda url=url, port=port: testConnectionFull(url=url, port=port)
 testAriaConnection = lambda url=url, port=port: testAriaConnectionFull(url=url, port=port)
+
